@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2004 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2014 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """Mixin class for MailList which handles administrative requests.
 
@@ -202,7 +202,7 @@ class ListAdmin:
                     cPickle.dump(msg, fp, 1)
                 else:
                     g = Generator(fp)
-                    g(msg, 1)
+                    g.flatten(msg, 1)
                 fp.flush()
                 os.fsync(fp.fileno())
             finally:
@@ -221,6 +221,8 @@ class ListAdmin:
         # an additional dictionary of message metadata
         #
         msgsubject = msg.get('subject', _('(no subject)'))
+        if not sender:
+            sender = _('<missing>')
         data = time.time(), sender, msgsubject, reason, filename, msgdata
         self.__db[id] = (HELDMSG, data)
         return id
@@ -241,7 +243,11 @@ class ListAdmin:
                 if e.errno <> errno.ENOENT: raise
                 return LOST
             try:
-                msg = cPickle.load(fp)
+                if path.endswith('.pck'):
+                    msg = cPickle.load(fp)
+                else:
+                    assert path.endswith('.txt'), '%s not .pck or .txt' % path
+                    msg = fp.read()
             finally:
                 fp.close()
             # Save the plain text to a .msg file, not a .pck file
@@ -250,8 +256,11 @@ class ListAdmin:
             outpath = head + '.msg'
             outfp = open(outpath, 'w')
             try:
-                g = Generator(outfp)
-                g(msg, 1)
+                if path.endswith('.pck'):
+                    g = Generator(outfp)
+                    g.flatten(msg, 1)
+                else:
+                    outfp.write(msg)
             finally:
                 outfp.close()
         # Now handle updates to the database
@@ -283,7 +292,8 @@ class ListAdmin:
             # message directly here can lead to a huge delay in web
             # turnaround.  Log the moderation and add a header.
             msg['X-Mailman-Approved-At'] = email.Utils.formatdate(localtime=1)
-            syslog('vette', 'held message approved, message-id: %s',
+            syslog('vette', '%s: held message approved, message-id: %s',
+                   self.internal_name(),
                    msg.get('message-id', 'n/a'))
             # Stick the message back in the incoming queue for further
             # processing.
@@ -292,9 +302,11 @@ class ListAdmin:
         elif value == mm_cfg.REJECT:
             # Rejected
             rejection = 'Refused'
+            lang = self.getMemberLanguage(sender)
+            subject = Utils.oneline(subject, Utils.GetCharSet(lang))
             self.__refuse(_('Posting of your message titled "%(subject)s"'),
                           sender, comment or _('[No reason given]'),
-                          lang=self.getMemberLanguage(sender))
+                          lang=lang)
         else:
             assert value == mm_cfg.DISCARD
             # Discarded
@@ -386,6 +398,7 @@ class ListAdmin:
                self.internal_name(), addr)
         # Possibly notify the administrator in default list language
         if self.admin_immed_notify:
+            i18n.set_language(self.preferred_language)
             realname = self.real_name
             subject = _(
                 'New subscription request to list %(realname)s from %(addr)s')
@@ -402,17 +415,22 @@ class ListAdmin:
             msg = Message.UserNotification(owneraddr, owneraddr, subject, text,
                                            self.preferred_language)
             msg.send(self, **{'tomoderators': 1})
+            # Restore the user's preferred language.
+            i18n.set_language(lang)
 
     def __handlesubscription(self, record, value, comment):
         stime, addr, fullname, password, digest, lang = record
         if value == mm_cfg.DEFER:
             return DEFER
         elif value == mm_cfg.DISCARD:
-            pass
+            syslog('vette', '%s: discarded subscription request from %s',
+                   self.internal_name(), addr)
         elif value == mm_cfg.REJECT:
             self.__refuse(_('Subscription request'), addr,
                           comment or _('[No reason given]'),
                           lang=lang)
+            syslog('vette', """%s: rejected subscription request from %s
+\tReason: %s""", self.internal_name(), addr, comment or '[No reason given]')
         else:
             # subscribe
             assert value == mm_cfg.SUBSCRIBE
@@ -460,9 +478,12 @@ class ListAdmin:
         if value == mm_cfg.DEFER:
             return DEFER
         elif value == mm_cfg.DISCARD:
-            pass
+            syslog('vette', '%s: discarded unsubscription request from %s',
+                   self.internal_name(), addr)
         elif value == mm_cfg.REJECT:
             self.__refuse(_('Unsubscription request'), addr, comment)
+            syslog('vette', """%s: rejected unsubscription request from %s
+\tReason: %s""", self.internal_name(), addr, comment or '[No reason given]')
         else:
             assert value == mm_cfg.UNSUBSCRIBE
             try:
@@ -499,7 +520,7 @@ class ListAdmin:
             subject = _('Request to mailing list %(realname)s rejected')
         finally:
             i18n.set_translation(otrans)
-        msg = Message.UserNotification(recip, self.GetBouncesEmail(),
+        msg = Message.UserNotification(recip, self.GetOwnerEmail(),
                                        subject, text, lang)
         msg.send(self)
 
@@ -538,7 +559,21 @@ class ListAdmin:
             except IOError, e:
                 if e.errno <> errno.ENOENT: raise
                 self.__db = {}
-        for id, (op, info) in self.__db.items():
+        for id, x in self.__db.items():
+            # A bug in versions 2.1.1 through 2.1.11 could have resulted in
+            # just info being stored instead of (op, info)
+            if len(x) == 2:
+                op, info = x
+            elif len(x) == 6:
+                # This is the buggy info. Check for digest flag.
+                if x[4] in (0, 1):
+                    op = SUBSCRIPTION
+                else:
+                    op = HELDMSG
+                self.__db[id] = op, x
+                continue
+            else:
+                assert False, 'Unknown record format in %s' % self.__filename
             if op == SUBSCRIPTION:
                 if len(info) == 4:
                     # pre-2.1a2 compatibility
@@ -553,7 +588,8 @@ class ListAdmin:
                     assert len(info) == 6, 'Unknown subscription record layout'
                     continue
                 # Here's the new layout
-                self.__db[id] = when, addr, fullname, passwd, digest, lang
+                self.__db[id] = op, (when, addr, fullname, passwd,
+                                     digest, lang)
             elif op == HELDMSG:
                 if len(info) == 5:
                     when, sender, subject, reason, text = info
@@ -562,7 +598,8 @@ class ListAdmin:
                     assert len(info) == 6, 'Unknown held msg record layout'
                     continue
                 # Here's the new layout
-                self.__db[id] = when, sender, subject, reason, text, msgdata
+                self.__db[id] = op, (when, sender, subject, reason,
+                                     text, msgdata)
         # All done
         self.__closedb()
 
