@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2005 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2014 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,7 +12,8 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+# USA.
 
 
 """Miscellaneous essential routines.
@@ -26,9 +27,9 @@ the mailing lists, and whatever else doesn't belong elsewhere.
 from __future__ import nested_scopes
 
 import os
+import sys
 import re
 import cgi
-import sha
 import time
 import errno
 import base64
@@ -52,6 +53,17 @@ from Mailman import mm_cfg
 from Mailman import Errors
 from Mailman import Site
 from Mailman.SafeDict import SafeDict
+from Mailman.Logging.Syslog import syslog
+
+try:
+    import hashlib
+    md5_new = hashlib.md5
+    sha_new = hashlib.sha1
+except ImportError:
+    import md5
+    import sha
+    md5_new = md5.new
+    sha_new = sha.new
 
 try:
     True, False
@@ -59,6 +71,13 @@ except NameError:
     True = 1
     False = 0
 
+try:
+    import dns.resolver
+    from dns.exception import DNSException
+    dns_resolver = True
+except ImportError:
+    dns_resolver = False
+
 EMPTYSTRING = ''
 UEMPTYSTRING = u''
 NL = '\n'
@@ -200,27 +219,43 @@ def LCDomain(addr):
 
 
 # TBD: what other characters should be disallowed?
-_badchars = re.compile(r'[][()<>|;^,\000-\037\177-\377]')
+_badchars = re.compile(r'[][()<>|:;^,\\"\000-\037\177-\377]')
+# Strictly speaking, some of the above are allowed in quoted local parts, but
+# this can open the door to certain web exploits so we don't allow them.
+# Only characters allowed in domain parts.
+_valid_domain = re.compile('[-a-z0-9]', re.IGNORECASE)
 
 def ValidateEmail(s):
-    """Verify that the an email address isn't grossly evil."""
+    """Verify that an email address isn't grossly evil."""
     # Pretty minimal, cheesy check.  We could do better...
     if not s or s.count(' ') > 0:
         raise Errors.MMBadEmailError
-    if _badchars.search(s) or s[0] == '-':
+    if _badchars.search(s):
         raise Errors.MMHostileAddress, s
     user, domain_parts = ParseEmail(s)
-    # This means local, unqualified addresses, are no allowed
+    # This means local, unqualified addresses, are not allowed
     if not domain_parts:
         raise Errors.MMBadEmailError, s
     if len(domain_parts) < 2:
         raise Errors.MMBadEmailError, s
+    # domain parts may only contain ascii letters, digits and hyphen
+    # and must not begin with hyphen.
+    for p in domain_parts:
+        if len(p) == 0 or p[0] == '-' or len(_valid_domain.sub('', p)) > 0:
+            raise Errors.MMHostileAddress, s
 
 
 
+# Patterns which may be used to form malicious path to inject a new
+# line in the mailman error log. (TK: advisory by Moritz Naumann)
+CRNLpat = re.compile(r'[^\x21-\x7e]')
+
 def GetPathPieces(envar='PATH_INFO'):
     path = os.environ.get(envar)
     if path:
+        if CRNLpat.search(path):
+            path = CRNLpat.split(path)[0]
+            syslog('error', 'Warning: Possible malformed path attack.')
         return [p for p in path.split('/') if p]
     return None
 
@@ -240,7 +275,7 @@ def ScriptURL(target, web_page_url=None, absolute=False):
         fullpath = os.environ.get('SCRIPT_NAME', '') + \
                    os.environ.get('PATH_INFO', '')
     baseurl = urlparse.urlparse(web_page_url)[2]
-    if not absolute and fullpath.endswith(baseurl):
+    if not absolute and fullpath.startswith(baseurl):
         # Use relative addressing
         fullpath = fullpath[len(baseurl):]
         i = fullpath.find('?')
@@ -325,7 +360,6 @@ def Secure_MakeRandomPassword(length):
                         # We have no available source of cryptographically
                         # secure random characters.  Log an error and fallback
                         # to the user friendly passwords.
-                        from Mailman.Logging.Syslog import syslog
                         syslog('error',
                                'urandom not available, passwords not secure')
                         return UserFriendly_MakeRandomPassword(length)
@@ -369,7 +403,7 @@ def set_global_password(pw, siteadmin=True):
     omask = os.umask(026)
     try:
         fp = open(filename, 'w')
-        fp.write(sha.new(pw).hexdigest() + '\n')
+        fp.write(sha_new(pw).hexdigest() + '\n')
         fp.close()
     finally:
         os.umask(omask)
@@ -395,12 +429,20 @@ def check_global_password(response, siteadmin=True):
     challenge = get_global_password(siteadmin)
     if challenge is None:
         return None
-    return challenge == sha.new(response).hexdigest()
+    return challenge == sha_new(response).hexdigest()
 
 
 
+_ampre = re.compile('&amp;((?:#[0-9]+|[a-z]+);)', re.IGNORECASE)
 def websafe(s):
-    return cgi.escape(s, quote=True)
+    if mm_cfg.BROKEN_BROWSER_WORKAROUND:
+        # Archiver can pass unicode here. Just skip them as the
+        # archiver escapes non-ascii anyway.
+        if isinstance(s, str):
+            for k in mm_cfg.BROKEN_BROWSER_REPLACEMENTS:
+                s = s.replace(k, mm_cfg.BROKEN_BROWSER_REPLACEMENTS[k])
+    # Don't double escape html entities
+    return _ampre.sub(r'&\1', cgi.escape(s, quote=True))
 
 
 def nntpsplit(s):
@@ -435,6 +477,9 @@ def UnobscureEmail(addr):
 
 
 
+class OuterExit(Exception):
+    pass
+
 def findtext(templatefile, dict=None, raw=False, lang=None, mlist=None):
     # Make some text from a template file.  The order of searches depends on
     # whether mlist and lang are provided.  Once the templatefile is found,
@@ -501,7 +546,6 @@ def findtext(templatefile, dict=None, raw=False, lang=None, mlist=None):
     searchdirs.append(os.path.join(mm_cfg.TEMPLATE_DIR, 'site'))
     searchdirs.append(mm_cfg.TEMPLATE_DIR)
     # Start scanning
-    quickexit = 'quickexit'
     fp = None
     try:
         for lang in languages:
@@ -509,12 +553,12 @@ def findtext(templatefile, dict=None, raw=False, lang=None, mlist=None):
                 filename = os.path.join(dir, lang, templatefile)
                 try:
                     fp = open(filename)
-                    raise quickexit
+                    raise OuterExit
                 except IOError, e:
                     if e.errno <> errno.ENOENT: raise
                     # Okay, it doesn't exist, keep looping
                     fp = None
-    except quickexit:
+    except OuterExit:
         pass
     if fp is None:
         # Try one last time with the distro English template, which, unless
@@ -540,7 +584,6 @@ def findtext(templatefile, dict=None, raw=False, lang=None, mlist=None):
                 text = sdict.interpolate(utemplate)
         except (TypeError, ValueError), e:
             # The template is really screwed up
-            from Mailman.Logging.Syslog import syslog
             syslog('error', 'broken template: %s\n%s', filename, e)
             pass
     if raw:
@@ -565,7 +608,7 @@ ADMINDATA = {
     'set':         (3, 3),
     'subscribe':   (0, 3),
     'unsubscribe': (0, 1),
-    'who':         (0, 0),
+    'who':         (0, 1),
     }
 
 # Given a Message.Message object, test for administrivia (eg subscribe,
@@ -665,6 +708,9 @@ def GetLanguageDescr(lang):
 def GetCharSet(lang):
     return mm_cfg.LC_DESCRIPTIONS[lang][1]
 
+def GetDirection(lang):
+    return mm_cfg.LC_DESCRIPTIONS[lang][2]
+
 def IsLanguage(lang):
     return mm_cfg.LC_DESCRIPTIONS.has_key(lang)
 
@@ -679,9 +725,11 @@ def get_domain():
     if mm_cfg.VIRTUAL_HOST_OVERVIEW and host:
         return host.lower()
     else:
-        # See the note in Defaults.py concerning DEFAULT_HOST_NAME
-        # vs. DEFAULT_EMAIL_HOST.
-        hostname = mm_cfg.DEFAULT_HOST_NAME or mm_cfg.DEFAULT_EMAIL_HOST
+        # See the note in Defaults.py concerning DEFAULT_URL
+        # vs. DEFAULT_URL_HOST.
+        hostname = ((mm_cfg.DEFAULT_URL
+                     and urlparse.urlparse(mm_cfg.DEFAULT_URL)[1])
+                     or mm_cfg.DEFAULT_URL_HOST)
         return hostname.lower()
 
 
@@ -777,12 +825,25 @@ def canonstr(s, lang=None):
     newparts = []
     parts = re.split(r'&(?P<ref>[^;]+);', s)
     def appchr(i):
-        if i < 256:
-            newparts.append(chr(i))
+        # do everything in unicode
+        newparts.append(unichr(i))
+    def tounicode(s):
+        # We want the default fallback to be iso-8859-1 even if the language
+        # is English (us-ascii).  This seems like a practical compromise so
+        # that non-ASCII characters in names can be used in English lists w/o
+        # having to change the global charset for English from us-ascii (which
+        # I superstitiously think may have unintended consequences).
+        if isinstance(s, unicode):
+            return s
+        if lang is None:
+            charset = 'iso-8859-1'
         else:
-            newparts.append(unichr(i))
+            charset = GetCharSet(lang)
+            if charset == 'us-ascii':
+                charset = 'iso-8859-1'
+        return unicode(s, charset, 'replace')
     while True:
-        newparts.append(parts.pop(0))
+        newparts.append(tounicode(parts.pop(0)))
         if not parts:
             break
         ref = parts.pop(0)
@@ -791,28 +852,16 @@ def canonstr(s, lang=None):
                 appchr(int(ref[1:]))
             except ValueError:
                 # Non-convertable, stick with what we got
-                newparts.append('&'+ref+';')
+                newparts.append(tounicode('&'+ref+';'))
         else:
             c = htmlentitydefs.entitydefs.get(ref, '?')
             if c.startswith('#') and c.endswith(';'):
                 appchr(int(ref[1:-1]))
             else:
-                newparts.append(c)
+                newparts.append(tounicode(c))
     newstr = EMPTYSTRING.join(newparts)
-    if isinstance(newstr, UnicodeType):
-        return newstr
-    # We want the default fallback to be iso-8859-1 even if the language is
-    # English (us-ascii).  This seems like a practical compromise so that
-    # non-ASCII characters in names can be used in English lists w/o having to
-    # change the global charset for English from us-ascii (which I
-    # superstitiously think may have unintended consequences).
-    if lang is None:
-        charset = 'iso-8859-1'
-    else:
-        charset = GetCharSet(lang)
-        if charset == 'us-ascii':
-            charset = 'iso-8859-1'
-    return unicode(newstr, charset, 'replace')
+    # newstr is unicode
+    return newstr
 
 
 # The opposite of canonstr() -- sorta.  I.e. it attempts to encode s in the
@@ -862,3 +911,243 @@ def oneline(s, cset):
     except (LookupError, UnicodeError, ValueError, HeaderParseError):
         # possibly charset problem. return with undecoded string in one line.
         return EMPTYSTRING.join(s.splitlines())
+
+
+# Patterns and functions to flag possible XSS attacks in HTML.
+# This list is compiled from information at http://ha.ckers.org/xss.html,
+# http://www.quirksmode.org/js/events_compinfo.html,
+# http://www.htmlref.com/reference/appa/events1.htm,
+# http://lxr.mozilla.org/mozilla/source/content/events/src/nsDOMEvent.cpp#59,
+# http://www.w3.org/TR/DOM-Level-2-Events/events.html and
+# http://www.xulplanet.com/references/elemref/ref_EventHandlers.html
+# Many thanks are due to Moritz Naumann for his assistance with this.
+_badwords = [
+    '<i?frame',
+    # Kludge to allow the specific tag that's in the options.html template.
+    '<link(?! rel="SHORTCUT ICON" href="<mm-favicon>">)',
+    '<meta',
+    '<object',
+    '<script',
+    r'\bj(?:ava)?script\b',
+    r'\bvbs(?:cript)?\b',
+    r'\bdomactivate\b',
+    r'\bdomattrmodified\b',
+    r'\bdomcharacterdatamodified\b',
+    r'\bdomfocus(?:in|out)\b',
+    r'\bdommenuitem(?:in)?active\b',
+    r'\bdommousescroll\b',
+    r'\bdomnodeinserted(?:intodocument)?\b',
+    r'\bdomnoderemoved(?:fromdocument)?\b',
+    r'\bdomsubtreemodified\b',
+    r'\bfscommand\b',
+    r'\bonabort\b',
+    r'\bon(?:de)?activate\b',
+    r'\bon(?:after|before)print\b',
+    r'\bon(?:after|before)update\b',
+    r'\bonbefore(?:(?:de)?activate|copy|cut|editfocus|paste)\b',
+    r'\bonbeforeunload\b',
+    r'\bonbegin\b',
+    r'\bonblur\b',
+    r'\bonbounce\b',
+    r'\bonbroadcast\b',
+    r'\bon(?:cell)?change\b',
+    r'\boncheckboxstatechange\b',
+    r'\bon(?:dbl)?click\b',
+    r'\bonclose\b',
+    r'\boncommand(?:update)?\b',
+    r'\boncomposition(?:end|start)\b',
+    r'\boncontextmenu\b',
+    r'\boncontrolselect\b',
+    r'\boncopy\b',
+    r'\boncut\b',
+    r'\bondataavailable\b',
+    r'\bondataset(?:changed|complete)\b',
+    r'\bondrag(?:drop|end|enter|exit|gesture|leave|over)?\b',
+    r'\bondragstart\b',
+    r'\bondrop\b',
+    r'\bonend\b',
+    r'\bonerror(?:update)?\b',
+    r'\bonfilterchange\b',
+    r'\bonfinish\b',
+    r'\bonfocus(?:in|out)?\b',
+    r'\bonhelp\b',
+    r'\boninput\b',
+    r'\bonkey(?:up|down|press)\b',
+    r'\bonlayoutcomplete\b',
+    r'\bon(?:un)?load\b',
+    r'\bonlosecapture\b',
+    r'\bonmedia(?:complete|error)\b',
+    r'\bonmouse(?:down|enter|leave|move|out|over|up|wheel)\b',
+    r'\bonmove(?:end|start)?\b',
+    r'\bon(?:off|on)line\b',
+    r'\bonoutofsync\b',
+    r'\bonoverflow(?:changed)?\b',
+    r'\bonpage(?:hide|show)\b',
+    r'\bonpaint\b',
+    r'\bonpaste\b',
+    r'\bonpause\b',
+    r'\bonpopup(?:hidden|hiding|showing|shown)\b',
+    r'\bonprogress\b',
+    r'\bonpropertychange\b',
+    r'\bonradiostatechange\b',
+    r'\bonreadystatechange\b',
+    r'\bonrepeat\b',
+    r'\bonreset\b',
+    r'\bonresize(?:end|start)?\b',
+    r'\bonresume\b',
+    r'\bonreverse\b',
+    r'\bonrow(?:delete|enter|exit|inserted)\b',
+    r'\bonrows(?:delete|enter|inserted)\b',
+    r'\bonscroll\b',
+    r'\bonseek\b',
+    r'\bonselect(?:start)?\b',
+    r'\bonselectionchange\b',
+    r'\bonstart\b',
+    r'\bonstop\b',
+    r'\bonsubmit\b',
+    r'\bonsync(?:from|to)preference\b',
+    r'\bonsyncrestored\b',
+    r'\bontext\b',
+    r'\bontimeerror\b',
+    r'\bontrackchange\b',
+    r'\bonunderflow\b',
+    r'\bonurlflip\b',
+    r'\bseeksegmenttime\b',
+    r'\bsvgabort\b',
+    r'\bsvgerror\b',
+    r'\bsvgload\b',
+    r'\bsvgresize\b',
+    r'\bsvgscroll\b',
+    r'\bsvgunload\b',
+    r'\bsvgzoom\b',
+    ]
+
+
+# This is the actual re to look for the above patterns
+_badhtml = re.compile('|'.join(_badwords), re.IGNORECASE)
+# This is used to filter non-printable us-ascii characters, some of which
+# can be used to break words to avoid recognition.
+_filterchars = re.compile('[\000-\011\013\014\016-\037\177-\237]')
+# This is used to recognize '&#' and '%xx' strings for _translate which
+# translates them to characters
+_encodedchars = re.compile('(&#[0-9]+;?)|(&#x[0-9a-f]+;?)|(%[0-9a-f]{2})',
+                           re.IGNORECASE)
+
+
+def _translate(mo):
+    """Translate &#... and %xx encodings into the encoded character."""
+    match = mo.group().lower().strip('&#;')
+    try:
+        if match.startswith('x') or match.startswith('%'):
+            val = int(match[1:], 16)
+        else:
+            val = int(match, 10)
+    except ValueError:
+        return ''
+    if val < 256:
+        return chr(val)
+    else:
+        return ''
+
+
+def suspiciousHTML(html):
+    """Check HTML string for various tags, script language names and
+    'onxxx' actions that can be used in XSS attacks.
+    Currently, this a very simple minded test.  It just looks for
+    patterns without analyzing context.  Thus, it potentially flags lots
+    of benign stuff.
+    Returns True if anything suspicious found, False otherwise.
+    """
+
+    if _badhtml.search(_filterchars.sub(
+                       '', _encodedchars.sub(_translate, html))):
+        return True
+    else:
+        return False
+
+
+# This takes an email address, and returns True if DMARC policy is p=reject
+# or possibly quarantine.
+def IsDMARCProhibited(mlist, email):
+    if not dns_resolver:
+         return False
+
+    email = email.lower()
+    at_sign = email.find('@')
+    if at_sign < 1:
+        return False
+    dmarc_domain = '_dmarc.' + email[at_sign+1:]
+
+    try:
+        resolver = dns.resolver.Resolver()
+        resolver.timeout = float(mm_cfg.DMARC_RESOLVER_TIMEOUT)
+        resolver.lifetime = float(mm_cfg.DMARC_RESOLVER_LIFETIME)
+        txt_recs = resolver.query(dmarc_domain, dns.rdatatype.TXT)
+    except (dns.resolver.NXDOMAIN, dns.resolver.NoAnswer):
+        return False
+    except DNSException, e:
+        syslog('error',
+               'DNSException: Unable to query DMARC policy for %s (%s). %s',
+              email, dmarc_domain, e.__class__)
+        return False
+    else:
+# people are already being dumb, don't trust them to provide honest DNS
+# where the answer section only contains what was asked for, nor to include
+# CNAMEs before the values they point to.
+        full_record = ""
+        results_by_name = {}
+        cnames = {}
+        want_names = set([dmarc_domain + '.'])
+        for txt_rec in txt_recs.response.answer:
+            if txt_rec.rdtype == dns.rdatatype.CNAME:
+                cnames[txt_rec.name.to_text()] = (
+                    txt_rec.items[0].target.to_text())
+            if txt_rec.rdtype != dns.rdatatype.TXT:
+                continue
+            results_by_name.setdefault(txt_rec.name.to_text(), []).append(
+                "".join(txt_rec.items[0].strings))
+        expands = list(want_names)
+        seen = set(expands)
+        while expands:
+            item = expands.pop(0)
+            if item in cnames:
+                if cnames[item] in seen:
+                    continue # cname loop
+                expands.append(cnames[item])
+                seen.add(cnames[item])
+                want_names.add(cnames[item])
+                want_names.discard(item)
+
+        if len(want_names) != 1:
+            syslog('error',
+                   """multiple DMARC entries in results for %s,
+                   processing each to be strict""",
+                   dmarc_domain)
+        for name in want_names:
+            if name not in results_by_name:
+                continue
+            dmarcs = filter(lambda n: n.startswith('v=DMARC1;'),
+                            results_by_name[name])
+            if len(dmarcs) == 0:
+                return False
+            if len(dmarcs) > 1:
+                syslog('error',
+                       """RRset of TXT records for %s has %d v=DMARC1 entries;
+                       testing them all""",
+                        dmarc_domain, len(dmarc))
+            for entry in dmarcs:
+                if re.search(r'\bp=reject\b', entry, re.IGNORECASE):
+                    syslog('vette',
+                        'DMARC lookup for %s (%s) found p=reject in %s = %s',
+                        email, dmarc_domain, name, entry)
+                    return True
+
+                if (mlist.dmarc_quarantine_moderation_action and
+                    re.search(r'\bp=quarantine\b', entry, re.IGNORECASE)):
+                    syslog('vette',
+                      'DMARC lookup for %s (%s) found p=quarantine in %s = %s',
+                            email, dmarc_domain, name, entry)
+                    return True
+
+    return False
+
